import os
import inspect
import colorama


def cprint(text, color='reset', verbose=True):
    """Print with colorama.

    text:       Text to be colored.
    color:      Color string or integer for foreground.
                Tuple of integers for both foreground and background.
    verbose:    Option to use this function as a verbose mode.
    """
    if verbose:
        colorama.init(autoreset=True)

        if type(color) is str:
            color = colorama.Fore.__dict__[color.upper()]
        elif type(color) is int:
            color = f"\x1b[38;5;{color}m"
        elif type(color) is tuple:
            color = f"\x1b[38;5;{color[0]};48;5;{color[1]}m"
            width, _ = os.get_terminal_size()
            text = f"{text}{' ' * (width - len(text))}"

        print(f"{color}{text}")
        colorama.deinit()


def doc(arg):
    """Docstring decorator.

    arg:    Docstring text or object.
    """
    def decorator(func):
        if type(arg) is str:
            func.__doc__ = arg
        elif inspect.isclass(arg):
            func.__doc__ = arg.__doc__
        else:
            func.__doc__ = None

        return func
    return decorator



import string
import pytest

import pyssword as m

# Run:
# $ pytest -rapP test_pyssword.py


def test_run_help(cli_invoker):
    result = cli_invoker(m.main, 'run -h'.split())
    assert result.exit_code == 0
    assert result.output.startswith('Usage: main run <options> <size>')


@pytest.mark.parametrize(
    'size', [
        *[pytest.param(i, marks=pytest.mark.xfail) for i in range(0, 10)],
        *range(10, 21),
    ]
)
def test_run_size(cli_invoker, size):
    result = cli_invoker(m.main, f'run {size}'.split())
    assert result.exit_code == 0
    assert len(result.output.strip()) == size


def test_run_full_punctuation(cli_invoker):
    alphabet = string.ascii_letters + string.digits + string.punctuation
    result = cli_invoker(m.main, 'run'.split())
    output = result.output.strip()

    assert result.exit_code == 0
    assert len(output) == 16
    assert not any(c not in alphabet for c in output), f"Invalid punctuations --> {tuple(c for c in output if c not in alphabet)} in {output}"


def test_run_small_punctuation(cli_invoker):
    alphabet = string.ascii_letters + string.digits + '!?@#$%&*+-_.,'
    result = cli_invoker(m.main, 'run -s'.split())
    output = result.output.strip()

    assert result.exit_code == 0
    assert len(output) == 16
    assert not any(c not in alphabet for c in output), f"Invalid punctuations --> {tuple(c for c in output if c not in alphabet)} in {output}"


def test_run_nopunctuation(cli_invoker):
    alphabet = string.ascii_letters + string.digits
    result = cli_invoker(m.main, 'run -n'.split())
    output = result.output.strip()

    assert result.exit_code == 0
    assert len(output) == 16
    assert not any(c not in alphabet for c in output), f"Invalid punctuations {tuple(c for c in output if c not in alphabet)} in {output}"


def test_run_small_and_nopunctuation(cli_invoker):
    alphabet = string.ascii_letters + string.digits
    result = cli_invoker(m.main, 'run -n -s'.split())
    output = result.output.strip()

    assert result.exit_code == 0
    assert len(output) == 16
    assert not any(c not in alphabet for c in output), f"Invalid punctuations {tuple(c for c in output if c not in alphabet)} in {output}"
